#include "common.h"

#define READ_END  0
#define WRITE_END 1

int bootZone( const char * );
void enterZone( const char * );
int getZoneState( const char *, char * );

int main(void) {
  char *reservedZoneName;
  char zoneState[30];

  init();
  reservedZoneName = getReservedZoneName();
  if( reservedZoneName == NULL ) {
    destroy();
    fprintf(stderr, "Error: you must run " WANT_ZONE_CMD " to reserve a zone.\n");
    return -1;
  }

  getZoneState( reservedZoneName, (char *)zoneState );
  printf("Zone state: %s\n", zoneState);

  if( strcmp( zoneState, "configured" ) == 0 ) {
    fprintf( stderr, "Error: zone is not installed. Please re-initialize zone.\n" );
    fprintf( stderr, "Hint: you might try " FREE_ZONE_CMD ";" WANT_ZONE_CMD "\n" );
  }

  if( strcmp( zoneState, "installed" ) == 0 ) {
    printf( "Zone is not running, bringing up zone...\n" );
    bootZone( reservedZoneName );

    // Get new zone status...
    getZoneState( reservedZoneName, (char *)zoneState );
    printf("Zone state: %s\n", zoneState);
  }

  // In theory, this is as far as this process will ever get.
  if( strcmp( zoneState, "running" ) == 0 ) {
    destroy();
    printf( "Entering zone...\n" );
    enterZone( reservedZoneName ); // will never return
  }

  if( strcmp( zoneState, "installed" ) == 0 ) {
    destroy();
    fprintf( stderr, "Error: could not boot zone.\n" );
    return -1;
  }

  fprintf( stderr, "Error: zone is in an unknown state: %s\n", zoneState ); 
  destroy();

  return -1;
}

int bootZone( const char *zoneName ) {
  int childPID;
  int exitStatus;

  childPID = fork();
  if( ! childPID ) { // child
    setreuid(0,0);

    // boot the zone
    execl("/usr/sbin/zoneadm", "zoneadm", "-z", zoneName, "boot", (char*)NULL);
    printf("WE ARE HERE!!!\n");
  }
  wait( &exitStatus );

  return 0;
}

void enterZone( const char *zoneName ) {
  setreuid(0,0);

  // enter the zone
  // TODO: add original program arguments to this exec function so
  //       that users can execute a function in the zone directly
  //       from the CLI
  execl("/usr/sbin/zlogin", "zlogin", zoneName, (char*)NULL);
}

int getZoneState( const char *zoneName, char * zoneState ) {
  int childWritePipe[2];  // Allow children to talk back to parent
  char readIn[1024]; // read in up to 1k char line
  int readBytes = 0;
  int childPID;
  int exitStatus;
  int i; // basic iteration

  zoneState[0] = '\0';

  // Allocate pipe for child->parent communication
  pipe(childWritePipe);

  childPID = fork();
  if( ! childPID ) { // child
    setreuid(0,0);
    // Redirect stdout to parent pipe
    close(fileno(stdout));
    dup2(childWritePipe[WRITE_END], fileno(stderr));
    dup2(childWritePipe[WRITE_END], fileno(stdout));

    // Get zone status
    execl("/usr/sbin/zoneadm", "zoneadm", "-z", zoneName, "list", "-p", (char*)NULL);
    printf("WE ARE HERE!!!\n");
  }
  wait( &exitStatus );

  // Get a line of data from the child process (kludgy but safe)
  while( readBytes >= 0 ) {
    readBytes += read(childWritePipe[READ_END], readIn + readBytes, 1);
    if( readBytes > 1024 ) {
      fprintf(stderr, "Error: line is too long.\n");
      return -1;
    }
    if( readBytes > 0 && readIn[readBytes-1] == '\n' ) { 
      readIn[readBytes-1] = '\0';
      readBytes = -1; // done
    }
  }

  // Check to see that there are 6 : delimeters in line
  readBytes = 0; // reuse readBytes to count : in line
  for( i = strlen(readIn) - 1; i >= 0 ; i-- ) {
    if( readIn[i] == ':' ) {
      readBytes++;
      if( readBytes == 4 ) 
        readIn[i] = '\0';
      else if( readBytes == 5 )
        memcpy(zoneState, readIn + i + 1, strlen( readIn + i + 1 ) + 1  );
    }
  }

  if( readBytes == 6 )
    return 0;
  return -1;

}
